<?php

namespace Teknisa\ACL\Util;

use Doctrine\DBAL\Connection;
use Zeedhi\Framework\DependencyInjection\InstanceManager;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO\Response\Error;
use Teknisa\ACL\Exception\CustomException;

class AclUtilities {
    const GERAL_ENTITY_DIR = '\Teknisa\Model\Geral\Entities\\';
    const ARRAY_OPERATOR = array("=","<>",">","<",">=","<=","IN","NOT_IN","BETWEEN","NOT_BETWEEN","LIKE","LIKE_I","NOT_LIKE","LIIKE_ALL","MAPPED_LIKE_ALL","IS_NULL","IS_NOT_NULL");
    const ARRAY_ORDER = array("DESC","ASC");

    public static function getBind($value,$colunm,$bindParams)
    {
        $colunm = preg_replace('/[^a-z0-9]/i', '_', $colunm);
        $colunmBind = ':FILTER_'.$colunm;
        if(empty($bindParams[$colunmBind])){
            $bindParams[$colunmBind] = $value;
        } else {
            list($colunmBind,$bindParams) = self::getBind($value,$colunm.'_',$bindParams);
        }
        return array($colunmBind,$bindParams);
    }

    public static function validateFilteredAndBind($name,$columnNamePermission,$operator)
    {
        $validade = true;
        if(!empty($columnNamePermission) ){
            if($operator == 'LIKE_ALL' || $operator == 'MAPPED_LIKE_ALL'){
                $columnList = explode('|', $name);
                foreach ($columnList as $column) {
                    if(!in_array($column,$columnNamePermission)) {
                        $validade = false;
                        throw New \Exception("Coluna $column não permitida para filtro");
                    }
                }
            } else if(!in_array($name,$columnNamePermission)) {
                $validade = false;
                throw New \Exception("Coluna $name não permitida para filtro");
            }
        } else {
            $validade = false;
            throw New \Exception("Array de coluna vazio");
        }
        if(!in_array($operator,self::ARRAY_OPERATOR)) {
            $validade = false;
            throw New \Exception("Operador $operator não permitido");
        }
        return $validade;
    }

    public static function validateOperator($operator)
    {
        $validade = true;
        if(!in_array($operator,self::ARRAY_OPERATOR)) {

        } else {
            $validade = false;
            throw New \Exception("Array de coluna vazio");
        }
        return $validade;
    }

    public static function createQueryFilteredAndBind($filters, $tableAlias = null,$columnNamePermission = array(),$bindParams = array() ,$typeBindParams = array())
    {
        $filterSQL = '';
        foreach ($filters as $filter) {
            $name = $filter['columnName'];
            $operator = $filter['operator'] == "LIKE_I" ? "LIKE" : $filter['operator'];
            $values = $filter['value'];
            if($name == 'NRLICENCAUSO'){
                $operator = 'LIKE';
            }
            if(self::validateFilteredAndBind($name,$columnNamePermission,$operator)) {
                list($bindParams, $filterSQL, $typeBindParams) = self::filterBind($name, $values, $operator, $bindParams, $tableAlias, $filterSQL, $typeBindParams);
            }
        }
        return array($filterSQL, $bindParams, $typeBindParams);
    }
    public static function createQueryFiltered($filters, $tableAlias = null)
    {
        $filterSQL = '';
        foreach ($filters as $filter) {
            $name = $filter['columnName'];
            $operator = $filter['operator'] == "LIKE_I" ? "LIKE" : $filter['operator'];
            $values = $filter['value'];
            if($name == 'NRLICENCAUSO'){
                $operator = 'LIKE';
            }
            //(!is_null($values) && $operator == '=') ->Gambi to fix framework bugs :(
            if ((($name != '*' && !empty($values)) || $values == 0) && !(empty($values) && $operator == '=') && !is_null($values)) {
                if ($operator == 'BETWEEN') {
                    if ($values[0] && $values[1]) {
                        $values = "TO_DATE( '$values[0] 00:00:00' ,'DD/MM/YYYY HH24:MI:SS')
                                AND TO_DATE( '$values[1] 23:59:59' ,'DD/MM/YYYY HH24:MI:SS')";
                    } else if ($values[0]) {
                        $operator = '>';
                        $values = "TO_DATE( '$values[0] 00:00:00' ,'DD/MM/YYYY HH24:MI:SS')";
                    } else if ($values[1]) {
                        $operator = '<';
                        $values = "TO_DATE( '$values[1] 00:00:00' ,'DD/MM/YYYY HH24:MI:SS')";
                    } else {
                        $values = "";
                    }

                    if ($values || ($values && ($values[0] || $values[1]))) {
                        if (!empty($tableAlias)) {
                            $filterSQL .= "AND $tableAlias.$name $operator $values";
                        } else {
                            $filterSQL .= "AND $name $operator $values";
                        }
                    }
                } else {
                    if (is_array($values)) {
                        if(is_array($values[0]) && isset($values[0]['_ALL'])) {
                            $values = implode(",", array_values($values[0]));
                        } else {
                            $values = "('" . implode("','", $values) . "')";
                        }
                    }
                    if ($operator == 'LIKE' || $operator == 'LIKE_I') {
                        $operator = 'LIKE';
                        if (!empty($tableAlias)) {
                            $values = "UPPER('" . $values . "')";
                            $name = "UPPER($tableAlias.$name)";
                        } else {
                            $values = "UPPER('" . $values . "')";
                            $name = "UPPER($name)";
                        }
                    }
                    if ($operator == 'LIKE_ALL' || $operator == 'MAPPED_LIKE_ALL') {
                        if (!empty($tableAlias)) {
                            $values = "UPPER('" . $values . "')";
                        } else {
                            $values = "UPPER('" . $values . "')";
                        }

                        $columnList = explode('|', $name);
                        $likes = array();
                        foreach ($columnList as $column) {
                            $likes[] = "UPPER($column) LIKE $values";
                        }
                        $filterLikeAll = implode(' OR ', $likes);
                        $filterSQL .= " AND ( $filterLikeAll )";
                    } else if ($operator == 'LIKE') {
                        $filterSQL .= " AND $name $operator $values ";
                    } else if ($operator == 'IS_NULL') {
                        if (!empty($tableAlias)) {
                            $filterSQL .= " AND $tableAlias.$name IS $values ";
                        } else {
                            $filterSQL .= " AND $name IS $values ";
                        }
                    } else if ($operator == '='){
                        if(!empty($tableAlias)){
                            $filterSQL .= " AND $tableAlias.$name $operator '$values' ";
                        } else {
                            $filterSQL .= " AND $name $operator '$values' ";
                        }
                    } else {
                        if (!empty($tableAlias)) {
                            $filterSQL .= " AND $tableAlias.$name $operator $values ";
                        } else {
                            $filterSQL .= " AND $name $operator $values ";
                        }
                    }
                }
            }
        }
        return $filterSQL;
    }
    
    public static function getValueInFilterDataSourceByName(array &$filters,$nameFilter,$remove = false)
    {
        foreach ($filters as $key => $filter) {
            $name = $filter['columnName'];
            $values = $filter['value'];
            if($name == $nameFilter ){
                if($remove){
                    unset($filters[$key]);
                }
                return $values;
            }
        }
        return null;
    }

    public static function getDefaultRuleParameter() {
        $defaultRule = null;

        try {
            return InstanceManager::getInstance()->getParameter('defaultRule');
        } catch(\Exception $e) { }

        return $defaultRule;
    }

    public static function getValidationNewCodeParameter() {
        $defaultRule = true;

        try {
            return InstanceManager::getInstance()->getParameter('validationNewCode');
        } catch(\Exception $e) { }

        return $defaultRule;
    }

    public static function getFilterTypeIn(array $items, $field)
    {
        return ' AND ' . $field . ' IN (' . implode(",", $items) . ')';
    }

    public static function createNewFilterCriteria($conditions, $dataSourceName) {
        $filterCriteria = new FilterCriteria($dataSourceName);
        foreach($conditions as $condition) {
            $name = isset($condition['name']) ? $condition['name'] : $condition['columnName'];
            if($condition['operator'] === 'ORDER_BY') {
                $filterCriteria->addOrderBy($name['name'], $name['direction']);
            } else {
                $filterCriteria->addCondition($name, $condition['operator'], $condition['value']);
            }
        }
        return $filterCriteria;
    }

    public static function fixBugWhereINQuery(array $ids, $column, $isNotIn = false, $isString = false)
    {
        $arrays = array_chunk($ids, 900);
        $allProjectsString = '';
        $in = $isNotIn ? 'NOT IN' : 'IN';
        $connection = $isNotIn ? 'AND' : 'OR';
        if ($isString) {
            $glue = "','";
            $start = "'";
            $end = "'";
        } else {
            $glue = ',';
            $start = '';
            $end = '';
        }
        foreach ($arrays as $array) {
            $allProjectsString .= " $column $in (" . self::getStringForArray($array, $glue, $start, $end) . ") $connection";
        }
        $allProjectsString .= " $column $in ($start 0 $end) ";
        return $allProjectsString;
    }

    public static function fixBugWhereINQueryBind($bindParams, $typeBindParams,array $ids, $column, $isNotIn = false, $isString = false)
    {

        $arrays = array_chunk($ids, 900);
        $allProjectsString = '';
        $in = $isNotIn ? 'NOT IN' : 'IN';
        $connection = $isNotIn ? 'AND' : 'OR';
        if ($isString) {
            $start = "'";
            $end = "'";
            $type = Connection::PARAM_STR_ARRAY;
        } else {
            $start = '';
            $end = '';
            $type = Connection::PARAM_INT_ARRAY;
        }
        foreach ($arrays as $array) {
            list($colunmBind, $bindParams) = AclUtilities::getBind($array, $column, $bindParams);
            $typeBindParams[$colunmBind] = $type;
            $allProjectsString .= " $column $in (" . $colunmBind . ") $connection";
        }
        $allProjectsString .= " $column $in ($start 0 $end) ";
        return array($bindParams, $typeBindParams, $allProjectsString);
    }

    public static function getStringForArray($array,$glue = ',',$start = '',$end = '',$default = ''){
        $return = $default;
        if(!empty($array)) {
            $return = $start . implode($glue, $array) . $end;
        }
        return $return;
    }

    public static function validadeOrder($idx,$order,array $fields)
    {
        $validade = true;
        if(!in_array($order,self::ARRAY_ORDER)) {
            $validade = false;
        }
        if(!in_array($idx,$fields)){
            $validade = false;
        }
        return $validade;

    }

    public static function getArrayFromAssociativeArray( array $array = array(), $key = 'ID')
    {
        $result = array();
        foreach ($array as $row) {
            $result[] = $row[$key] ;
        }
        return $result;
    }
    
    public static function gerarDescricao($values, $nomesDosCampos) {
        $descricao = '';
        for ($i = 0; $i < count($values); $i++) {
            $descricao .= $nomesDosCampos[$i] . " = " . $values[$i] . "\n";
        }
        return $descricao;
    }

    /**
     * @param CustomException $e
     * @return Error
     */
    public static function getErrorObjectFromException(CustomException $e, $message = null) {
        $error = array (
            'CODE'   => $message ?: $e->getMessage(),
            'PARAMS' => $e->getParams()
        );
        return new Error(json_encode($error), $e->getCode(), $e->getTraceAsString());
    }

    /**
     * @param $name
     * @param $values
     * @param $operator
     * @param $bindParams
     * @param $tableAlias
     * @param string $filterSQL
     * @param $typeBindParams
     * @return array
     */
    private static function filterBind($name, $values, $operator, $bindParams, $tableAlias, string $filterSQL, $typeBindParams): array
    {
//(!is_null($values) && $operator == '=') ->Gambi to fix framework bugs :(
        if ((($name != '*' && !empty($values)) || $values == 0) && !(empty($values) && $operator == '=') && !is_null($values)) {
            if ($operator == 'BETWEEN') {
                if ($values[0] && $values[1]) {
                    list($colunmBind, $bindParams) = self::getBind($values[0], $name, $bindParams);
                    list($colunmBind2, $bindParams) = self::getBind($values[1], $name, $bindParams);
                    $values = "TO_DATE( $colunmBind||' 00:00:00' ,'DD/MM/YYYY HH24:MI:SS')
                                AND TO_DATE( $colunmBind2||' 23:59:59' ,'DD/MM/YYYY HH24:MI:SS')";
                } else if ($values[0]) {
                    list($colunmBind, $bindParams) = self::getBind($values[0], $name, $bindParams);
                    $operator = '>';
                    $values = "TO_DATE( $colunmBind||' 00:00:00' ,'DD/MM/YYYY HH24:MI:SS')";
                } else if ($values[1]) {
                    list($colunmBind, $bindParams) = self::getBind($values[1], $name, $bindParams);
                    $operator = '<';
                    $values = "TO_DATE( $colunmBind||' 00:00:00' ,'DD/MM/YYYY HH24:MI:SS')";
                } else {
                    $values = "";
                }

                if ($values || ($values && ($values[0] || $values[1]))) {
                    if (!empty($tableAlias)) {
                        $filterSQL .= "AND $tableAlias.$name $operator $values";
                    } else {
                        $filterSQL .= "AND $name $operator $values";
                    }
                }
            } else {
                if (is_array($values)) {
                    if (is_array($values[0]) && isset($values[0]['_ALL'])) {
                        list($colunmBind, $bindParams) = self::getBind($values[0]['_ALL'], 'LIKE_ALL', $bindParams);
                        $values = $colunmBind;
                    } else {
                        list($colunmBind, $bindParams) = self::getBind($values, $name, $bindParams);
                        $values = "(" . $colunmBind . ")";
                        $typeBindParams[$colunmBind] = Connection::PARAM_STR_ARRAY;
                    }
                } else {
                    list($colunmBind, $bindParams) = self::getBind($values, $name, $bindParams);
                    $values = $colunmBind;
                }
                if ($operator == 'LIKE' || $operator == 'LIKE_I') {
                    $operator = 'LIKE';
                    if (!empty($tableAlias)) {
                        $values = "UPPER(" . $values . ")";
                        $name = "UPPER($tableAlias.$name)";
                    } else {
                        $values = "UPPER(" . $values . ")";
                        $name = "UPPER($name)";
                    }
                }
                if ($operator == 'LIKE_ALL' || $operator == 'MAPPED_LIKE_ALL') {
                    $values = "UPPER(" . $values . ")";
                    $columnList = explode('|', $name);
                    $likes = array();
                    foreach ($columnList as $column) {
                        $likes[] = "UPPER($column) LIKE $values";
                    }
                    $filterLikeAll = implode(' OR ', $likes);
                    $filterSQL .= " AND ( $filterLikeAll )";
                } else if ($operator == 'LIKE') {
                    $filterSQL .= " AND $name $operator $values ";
                } else if ($operator == 'IS_NULL') {
                    if (!empty($tableAlias)) {
                        $filterSQL .= " AND $tableAlias.$name IS $values ";
                    } else {
                        $filterSQL .= " AND $name IS $values ";
                    }
                } else if ($operator == '=') {
                    if (!empty($tableAlias)) {
                        $filterSQL .= " AND $tableAlias.$name $operator $values'";
                    } else {
                        $filterSQL .= " AND $name $operator $values ";
                    }
                } else {
                    if (!empty($tableAlias)) {
                        $filterSQL .= " AND $tableAlias.$name $operator $values ";
                    } else {
                        $filterSQL .= " AND $name $operator $values ";
                    }
                }
            }
        }
        return array($bindParams, $filterSQL, $typeBindParams);
    }
} 